const express = require("express");
const bcrypt = require("bcrypt");
const {addUser, userExists, getUser } = require("../db.js");

const router = express.Router();

router.get('/favicon.ico', (req, res) => res.status(204));

router.post("/sign-up", (req, res) => {
    const { username, password } = req.body;

    // Check if the email is already in use
    if (userExists(username)) {
        res.status(401).send({ message: "Username is already in use"});
        return;
    }

    // Define salt rounds, higher values increase the difficulty of
    // brute forcing but drastically increases hash times
    const saltRounds = 10

    // Hash the password
    bcrypt.hash(password, saltRounds, (err, hash) => {
        if (err) throw new Error("Internal Server Error")

        // Create user
        let user = {
            username,
            password: hash
        }

        // Save user to database
        addUser(user)
        res.json({ message: "User created successfully" })
    })
})

router.post("/sign-in", async (req, res) => {
    // Extract email and password from the req.body object
    const { username, password } = req.body

    // Get user if it exists in database
    const user = getUser(username)

    if (!user) {
        return res.status(401).json({ message: "Invalid credentials: The user does not exist." })
    }

    if(req.session.loggedin){
        return res.status(401).json({ message: "A user is already logged in" })
    }
    
    // Compare passwords
    bcrypt.compare(password, user.password, (err, result) => {
        if (result) {
            // update session to indicate that user is logged in
            req.session.loggedin = true;
            req.session.username = username;
            return res.status(200).json({ message: "User logged in successfully" })
        }
        return res.status(401).json({ message: "Invalid credentials: Wrong username or password." })
    })
})

router.post('/status',(req,res) => {
    let msg;
    if(req.session.loggedin){
        msg = `Logged in as ${req.session.username}`
    } else {
        msg = 'Not logged in'
    }
    res.status(201).send({ message: msg})
});

router.post('/logout',(req,res) => {
    if(req.session.loggedin){
        req.session.destroy();
        res.status(201).send({ message: 'Successfully logged out'})
    } else {
        res.status(201).send({ message: 'Not logged in'})
    }
    
});
   
  module.exports = router